/*
 *  mdlhtml.c
 *  delve
 *
 *  Created by timo on 02/01/2011.
 *  Copyright 2011 RIKEN Yokohama Institute, Genome Exploration Research Group. All rights reserved.
 *
 */
#include "delve.h"
#include "interface.h"

#include "2ndorder.h"
#include "mdlhtml.h"
#include "mathsupport.h"

void make_mdl_html(struct parameters* param )
{
	
	struct hmm2* hmm = init_2nd_hmm(500, 500);
	//int r1,g1,r2,g2,i;
	param->hmm_file = param->infile[1];
	read_hmm2_from_file(hmm,param);
	
	
	print_header(param->hmm_file);
	print_model_table(hmm);
	print_footer();
	free_2nd_hmm(hmm);
}


void print_header(char* lib_name)
{
	fprintf(outfile,"<!DOCTYPE html>\n");
	fprintf(outfile,"<head>\n");
	fprintf(outfile,"<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"/> \n");
	fprintf(outfile,"<title>Library %s</title>\n",shorten_pathname(lib_name));
	fprintf(outfile,"<meta name=\"keyword\" content=\"html5 library information\" />\n");
	fprintf(outfile,"<meta name=\"description\" content=\"site for experimenting with the features of html5\" />\n");
	fprintf(outfile,"<!--[if IE]>\n");
	fprintf(outfile,"<script src=\"http://html5shiv.googlecode.com/svn/trunk/html5.js\"></script>\n");
	fprintf(outfile,"<script src=\"j/excanvas.js\"></script>\n");
	fprintf(outfile,"<![endif]-->\n");
	fprintf(outfile,"<style>\n");
	fprintf(outfile,"#graphSpace { margin-left:80px; }\n");
	fprintf(outfile,"body { margin: 20px; font-family: arial,verdana,helvetica; background: #fff;}\n");
	fprintf(outfile,"h1 {color: rgb(36,89,158)}\n");
	fprintf(outfile,"h2 { clear:both;font-size: 140%%; font-weight:normal;  border-bottom: 1px solid rgb(0,0,0); }\n");
	fprintf(outfile,"canvas { margin-right: 20px; margin-bottom: 20px; }\n");
	fprintf(outfile,"pre { float : right; display:block; background: rgb(238,238,238); border: 1px dashed #666; padding: 15px 20px; margin: 0 0 10px 0;width:20%%;white-space:normal; }\n");
	fprintf(outfile,"help {float:right;	margin:0px;	width:auto;	height:auto;	border:1px dashed #66;	overflow:hidden;	background: rgb(238,238,238);padding: 1px;}\n");
	//fprintf(outfile,"table {border-collapse: collapse;float:left;margin-bottom:20px;}\n");
	fprintf(outfile,"table { border-collapse: collapse; border: solid thick }\n");
	fprintf(outfile,"colgroup, tbody { border: solid medium; }\n");
	//fprintf(outfile,"#merlist {border: 2px solid #000000; border-collapse: collapse;float:none;}\n");
	//fprintf(outfile," td {border: 1px solid #000000;width:50px;padding:5px;font-family:\"Courier New\" Courier 	monospace;font-size: 10pt;}\n");
	//fprintf(outfile,"#merlist th {border: 2px solid #000000;width:auto;padding:5px;}\n");
	fprintf(outfile,"td { border: solid thin; height: 1.4em; width: 1.4em; text-align: center; padding: 0;font-family:\"Courier New\" Courier 	monospace;font-size: 10pt; }\n");
	
	//fprintf(outfile,"tr {text-align:right; font:  normal 10pt Arial;}\n");
	//fprintf(outfile,"th {text-align:center;}\n");
	//fprintf(outfile,"tr.top  {border-top: 2px solid #000000;}\n");
	
	
	
	
	//fprintf(outfile,"table.mers {border-collapse:collapse;}\n");
	//fprintf(outfile,"table.mers td  {width:auto;padding:5px;font-family:\"Courier New\" Courier 	monospace;font-size: 12pt;}\n");
	//fprintf(outfile,"table.mers th  {width:auto;padding:5px;font-family:\"Courier New\" Courier 	monospace;font-size: 12pt;}\n");
	
	
	//fprintf(out,"table.mers.th, table.mers.th  {font-family:\"Courier New\" Courier 	monospace;font-size: 12pt;}\n");
	
	
	
	fprintf(outfile,"</style>\n");
	fprintf(outfile,"</head>\n");
	fprintf(outfile,"<body>\n");
	fprintf(outfile,"<h1>%s</h1>\n",shorten_pathname(lib_name));

}

void print_footer()
{
	
	fprintf(outfile,"<br /><footer>Created by %s. Contact: timolassmann@gmail.com</footer>",VERSION);
	
	fprintf(outfile,"</body>\n");
	fprintf(outfile,"</html>\n");
}


void print_model_table(struct hmm2* hmm )
{
	int r1,g1,r2,g2;//,i;
	int old_r1,old_g1;
	char alphabet[] = "ACGTactgACGTN-.";
	char alphabetg[] = "ACGTN-.";
	float tmp,sum;
	fprintf(outfile,"<section>\n");
	fprintf(outfile,"<h2>Nucleotide Over-representation along read</h2>\n");
	fprintf(outfile,"<table>\n");
	
	
	old_r1 = -1;
	old_g1 = -1;
	
	
	for(r1 = 0; r1 < 15;r1++){
		if(r1 != old_r1){
			fprintf(outfile,"<tr><td >");
			for(r2 = 0; r2 < 15;r2++){
				for(g2 = 0;g2 < 6;g2++){
					
					fprintf(outfile,"<td>%c<br>%c",alphabet[r2],alphabetg[g2]  );
					
				}
			}
			fprintf(outfile,"<td>\n");
			fprintf(outfile,"<td>\n");
		}
		
		
		for(g1 = 0;g1 < 6;g1++){
			fprintf(outfile,"<tr class=\"top\" >");
			
			fprintf(outfile,"<td>%c<br>%c",alphabet[r1],alphabetg[g1] );
			sum = 0;
			for(r2 = 0; r2 < 15;r2++){
				for(g2 = 0;g2 < 6;g2++){
					tmp = scaledprob2prob(hmm->prob[(r1 << 12) | (g1 << 8) | (r2 << 4) | g2   ] )* 1000.0f;
					if(tmp >= 300){
						fprintf(outfile,"<td style = \"background: #%06x;  \" >\n",red);
					}else if (tmp >= 200){
						fprintf(outfile,"<td style = \"background: #%06x;  \" >\n",orange);
					}else if (tmp >= 100){
						fprintf(outfile,"<td style = \"background: #%06x;  \" >\n",lemon);
					}else if (tmp >= 50){
						fprintf(outfile,"<td style = \"background: #%06x;  \" >\n",green);
					}else if (tmp >= 10){
						fprintf(outfile,"<td style = \"background: #%06x;  \" >\n",lightblue);
					}else{
						fprintf(outfile,"<td style = \"background: #%06x;  \" >\n",lightgrey);
					}
					sum += tmp;
					fprintf(outfile,"%4.0f",tmp);
				}
				
			}
			fprintf(outfile,"<td>%f",sum);
			fprintf(outfile,"<td>%c<br>%c",alphabet[r1],alphabetg[g1]  );
			fprintf(outfile,"\n");
		}
		
		
		
		old_r1 = r1;
		
		
	}
	/*
	fprintf(outfile,"<tr><td >");
	for(r2 = 0; r2 < 15;r2++){
		for(g2 = 0;g2 < 4;g2++){
			
			if(r2 < 12){
				if(r2 < 4){
					fprintf(outfile,"<td>%c<br>%c",alphabet[r2 & 0x3],alphabet[g2]  );
				}else if(r2 < 8){
					fprintf(outfile,"<td>e%c<br>%c",alphabet[r2 & 0x3],alphabet[g2]  );
				}else if(r2 < 12){
					fprintf(outfile,"<td>s%c<br>%c",alphabet[r2 & 0x3],alphabet[g2]  );
				}
			}else if(r2 == 12){
				fprintf(outfile,"<td>N<br>%c",alphabet[g2]);
			}else if(r2 == 13) {
				fprintf(outfile,"<td>-<br>%c",alphabet[g2]);
			}else if(r2 == 14) {
				fprintf(outfile,"<td>_<br>%c",alphabet[g2]);
			}
		}
	}
	fprintf(outfile,"<td>\n");
	for(r1 = 0; r1 < 15;r1++){
		
		
		
		g1 = 12;
		fprintf(outfile,"<tr class=\"top\" >");
		if(r1 < 12){
			if(r1 < 4){
				fprintf(outfile,"<td>%c<br>N",alphabet[r1 & 0x3] );
			}else if(r1 < 8){
				fprintf(outfile,"<td>e%c<br>N",alphabet[r1 & 0x3] );
			}else if(r1 < 12){
				fprintf(outfile,"<td>s%c<br>N",alphabet[r1 & 0x3]);
			}
		}else if(r1 == 12){
			fprintf(outfile,"<td>N<br>N");
		}else if(r1 == 13) {
			fprintf(outfile,"<td>-<br>N");
		}else if(r1 == 14) {
			fprintf(outfile,"<td>_<br>N");
		}
		for(r2 = 0; r2 < 15;r2++){
			for(g2 = 0;g2 < 4;g2++){
				tmp = scaledprob2prob(hmm->prob[(r1 << 12) | (g1 << 8) | (r2 << 4) | g2   ] )* 1000.0f;
				if(tmp >= 300){
					fprintf(outfile,"<td style = \"background: #%06x;  \" >\n",red);
				}else if (tmp >= 200){
					fprintf(outfile,"<td style = \"background: #%06x;  \" >\n",orange);
				}else if (tmp >= 100){
					fprintf(outfile,"<td style = \"background: #%06x;  \" >\n",lemon);
				}else if (tmp >= 50){
					fprintf(outfile,"<td style = \"background: #%06x;  \" >\n",green);
				}else if (tmp >= 10){
					fprintf(outfile,"<td style = \"background: #%06x;  \" >\n",lightblue);
				}else{
					fprintf(outfile,"<td style = \"background: #%06x;  \" >\n",lightgrey);
				}
				
				fprintf(outfile,"%4.0f",tmp);
			}
		}
		if(r1 < 12){
			if(r1 < 4){
				fprintf(outfile,"<td>%c<br>N",alphabet[r1 & 0x3] );
			}else if(r1 < 8){
				fprintf(outfile,"<td>e%c<br>N",alphabet[r1 & 0x3] );
			}else if(r1 < 12){
				fprintf(outfile,"<td>s%c<br>N",alphabet[r1 & 0x3]);
			}
		}else if(r1 == 12){
			fprintf(outfile,"<td>N<br>N");
		}else if(r1 == 13) {
			fprintf(outfile,"<td>-<br>N");
		}else if(r1 == 14) {
			fprintf(outfile,"<td>_<br>N");
		}
		//r1 = (i >> 12) & 15;
		//g1= (i >> 8) & 15;
		//r2 = (i >> 4) & 15;
		//g2 = i & 15; 
		fprintf(outfile,"\n");
	}*/
	/*fprintf(outfile,"<tr><td >");
	for(r2 = 0; r2 < 15;r2++){
		for(g2 = 0;g2 < 4;g2++){
			
			if(r2 < 12){
				if(r2 < 4){
					fprintf(outfile,"<td>d%c<br>%c",alphabet[r2 & 0x3],alphabet[g2]  );
				}else if(r2 < 8){
					fprintf(outfile,"<td>e%c<br>%c",alphabet[r2 & 0x3],alphabet[g2]  );
				}else if(r2 < 12){
					fprintf(outfile,"<td>s%c<br>%c",alphabet[r2 & 0x3],alphabet[g2]  );
				}
			}else if(r2 == 12){
				fprintf(outfile,"<td>N<br>%c",alphabet[g2]);
			}else if(r2 == 13) {
				fprintf(outfile,"<td>-<br>%c",alphabet[g2]);
			}else if(r2 == 14) {
				fprintf(outfile,"<td>_<br>%c",alphabet[g2]);
			}
		}
	}
	fprintf(outfile,"<td >\n");
	for(r1 = 0; r1 < 15;r1++){
		g1 = 13;
		fprintf(outfile,"<tr class=\"top\" >");
		if(r1 < 12){
			if(r1 < 4){
				fprintf(outfile,"<td>d%c<br>-",alphabet[r1 & 0x3] );
			}else if(r1 < 8){
				fprintf(outfile,"<td>e%c<br>-",alphabet[r1 & 0x3] );
			}else if(r1 < 12){
				fprintf(outfile,"<td>s%c<br>-",alphabet[r1 & 0x3]);
			}
		}else if(r1 == 12){
			fprintf(outfile,"<td>N<br>-");
		}else if(r1 == 13) {
			fprintf(outfile,"<td>-<br>-");
		}else if(r1 == 14) {
			fprintf(outfile,"<td>_<br>-");
		}
		for(r2 = 0; r2 < 15;r2++){
			for(g2 = 0;g2 < 4;g2++){
				tmp = scaledprob2prob(hmm->prob[(r1 << 12) | (g1 << 8) | (r2 << 4) | g2   ] )* 1000.0f;
				if(tmp >= 300){
					fprintf(outfile,"<td style = \"background: #%06x;  \" >\n",red);
				}else if (tmp >= 200){
					fprintf(outfile,"<td style = \"background: #%06x;  \" >\n",orange);
				}else if (tmp >= 100){
					fprintf(outfile,"<td style = \"background: #%06x;  \" >\n",lemon);
				}else if (tmp >= 50){
					fprintf(outfile,"<td style = \"background: #%06x;  \" >\n",green);
				}else if (tmp >= 10){
					fprintf(outfile,"<td style = \"background: #%06x;  \" >\n",lightblue);
				}else{
					fprintf(outfile,"<td style = \"background: #%06x;  \" >\n",lightgrey);
				}
				
				fprintf(outfile,"%4.0f",tmp);
			}
		}
		if(r1 < 12){
			if(r1 < 4){
				fprintf(outfile,"<td>d%c<br>-",alphabet[r1 & 0x3] );
			}else if(r1 < 8){
				fprintf(outfile,"<td>e%c<br>-",alphabet[r1 & 0x3] );
			}else if(r1 < 12){
				fprintf(outfile,"<td>s%c<br>-",alphabet[r1 & 0x3]);
			}
		}else if(r1 == 12){
			fprintf(outfile,"<td>N<br>-");
		}else if(r1 == 13) {
			fprintf(outfile,"<td>-<br>-");
		}else if(r1 == 14) {
			fprintf(outfile,"<td>_<br>-");
		}
		//r1 = (i >> 12) & 15;
		//g1= (i >> 8) & 15;
		//r2 = (i >> 4) & 15;
		//g2 = i & 15; 
		fprintf(outfile,"\n");
	}
	*/
	
	
	fprintf(outfile,"</table>\n");
	fprintf(outfile,"</section>\n");
}


char* shorten_pathname(char* p)
{
	int i;
	char* tmp = p;
	for(i = 0; i< strlen(p);i++){
		if(p[i] == '/'){
			tmp = p+i +1;
		}
	}
	return tmp;
}

